e = 1.602e-19;% electron charge
eps0 = 8.85e-12;
% % % % % 
%Inputs
% % % % % 
N = 50;   % Truncation order: Conevergence parameter. 
L = 8e-6; % Period
W = 0.5*L; % Ribbons width
Ef = 0.6*e; % Fermi energy
tau = .25e-12; % Relaxation time
lambda = lambdar/L; % Free space wavelength normalized to the grating period
theta = 0*pi/180; % incident angle
n1 = 3^0.5;   % Refractive index of incident medium
n2 = 4^0.5; % Refractive index of transmission medium
% -----------------------------------
% Defining grating region parameteres
% -----------------------------------
% Transition (discontinuity) points in profile of refractive index are
% defined by 'xl'. You must set first and last element of 'xl' to 0 and 1,
% repectively. It means that the first transition occurs at the begining of
% the grating period. 
xl = [0 W/L 1]; 

G0 = conductivity(Ef,tau,lambda*L);

k0=2*pi/lambda;
omega = k0*(3e8);
h = min([W,L-W,lambdar/5])/L/invh;
% The conductivity between transitions is determined by 'Gl'. 'Gl(m)' 
% is the conductivity of the region between xl(m) and xl(m+1).

Gl = [G0 0]+1i*omega*eps0*(n1^2+n2^2)*h/2;
% -------------------
% Initial computaions
% -------------------
k0=2*pi/lambda;
omega = k0*(3e8);
eps0 = 8.85e-12;
eps1 = eps0*n1^2;
eps2 = eps0*n2^2;

k1=k0*n1;
k2=k0*n2;
kx=zeros(2*N+1,1);
kz1 = kx;
kz2= kx;
kx(1:2*N+1)=2*pi*(n1*sin(theta)/lambda-(-N:N));
kz1(1:2*N+1)=sqrt(k1^2-kx(1:2*N+1).^2);
kz2(1:2*N+1)=sqrt(k2^2-kx(1:2*N+1).^2);
for j=1:2*N+1
    if -real(kz1(j))>1e-12
        kz1(j)=-kz1(j);
    end
    if imag(kz1(j))>1e-12
        kz1(j)=-kz1(j);
    end
    
    if -real(kz2(j))>1e-12
        kz2(j)=-kz2(j);
    end
    if imag(kz2(j))>1e-12
        kz2(j)=-kz2(j);
    end
end
Kx=diag(kx);
Kz1=diag(kz1);
Kz2=diag(kz2);
exph1=diag(exp(1i*kz1*h/2));
invexph1=diag(exp(-1i*kz1*h/2));
exph2=diag(exp(1i*kz2*h/2));
invexph2=diag(exp(-1i*kz2*h/2));
I=eye(2*N+1);
% --------------------

% ------------------------------------------
% Construction of required toeplitz matrices
% ------------------------------------------
lxl = length(xl);
gmn=fastftptrain(xl(2:(lxl-1)),1./Gl,2*N);
Gmn=inv(toeplitz(gmn(2*N+1:4*N+1),gmn(2*N+1:-1:1)));
% ------------------------------------------
% --------------------
% S matrix calculation
% --------------------
eta1 = (Kz1./(omega*eps1));
eta2 = (Kz2./(omega*eps2));
zayp = eta2*Gmn;

s1=[eta2 eta1; -zayp-invexph2 invexph1]\[eta1 eta2;-exph1 -zayp+exph2];


S11=s1(1:(2*N+1),1:(2*N+1));
S12=s1(1:(2*N+1),(2*N+2):(4*N+2));
S21=s1((2*N+2):(4*N+2),1:(2*N+1));
S22=s1((2*N+2):(4*N+2),(2*N+2):(4*N+2));
% ------------------

% Reflection and transmission coefficient

incident=zeros(2*N+1,1);
incident(N+1)=1;
Ttm = S11*incident;
Rtm = S21*incident;

% Diffraction efficiencies
DE1=zeros(1,2*N+1);
DE2=zeros(1,2*N+1);

for j=1:2*N+1
    DE1(j)=(abs(Rtm(j))^2)*real(kz1(j)/k1/cos(theta));
    DE2(j)=(abs(Ttm(j))^2)*real(n1^2*kz2(j)/(k1*(n2^2)*cos(theta)));
end
1-sum(DE1)-sum(DE2)